/*******************************************************************************
 * Copyright (c) 2015-2018 Skymind, Inc.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Apache License, Version 2.0 which is available at
 * https://www.apache.org/licenses/LICENSE-2.0.
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations
 * under the License.
 *
 * SPDX-License-Identifier: Apache-2.0
 ******************************************************************************/

package org.datavec.audio.recordreader;

import org.bytedeco.javacv.FFmpegFrameGrabber;
import org.bytedeco.javacv.Frame;
import org.datavec.api.writable.FloatWritable;
import org.datavec.api.Writable;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.nio.FloatBuffer;
import java.util.ArrayList;
import java.util.List;

import static org.bytedeco.ffmpeg.global.avutil.AV_SAMPLE_FMT_FLT;

/**
 * Native audio file loader using FFmpeg.
 *
 * @author saudet
 */
public class NativeAudioRecordReader extends BaseAudioRecordReader {

    public NativeAudioRecordReader() {}

    public NativeAudioRecordReader(boolean appendLabel, List<String> labels) {
        super(appendLabel, labels);
    }

    public NativeAudioRecordReader(List<String> labels) {
        super(labels);
    }

    public NativeAudioRecordReader(boolean appendLabel) {
        super(appendLabel);
    }

    protected List<Writable> loadData(File file, InputStream inputStream) throws IOException {
        List<Writable> ret = new ArrayList<>();
        try (FFmpegFrameGrabber grabber = inputStream != null ? new FFmpegFrameGrabber(inputStream)
                        : new FFmpegFrameGrabber(file.getAbsolutePath())) {
            grabber.setSampleFormat(AV_SAMPLE_FMT_FLT);
            grabber.start();
            Frame frame;
            while ((frame = grabber.grab()) != null) {
                while (frame.samples != null && frame.samples[0].hasRemaining()) {
                    for (int i = 0; i < frame.samples.length; i++) {
                        ret.add(new FloatWritable(((FloatBuffer) frame.samples[i]).get()));
                    }
                }
            }
        }
        return ret;
    }

}
